@@ -5,23 +5,21 @@
  *
  * The library is free for all purposes without any express
  * guarantee it works.
- *
- * Tom St Denis, tomstdenis@gmail.com, http://libtomcrypt.com
  */
 #include "tomcrypt.h"
 
 /**
   @file hmac_init.c
-  HMAC support, initialize state, Tom St Denis/Dobes Vandermeer 
+  HMAC support, initialize state, Tom St Denis/Dobes Vandermeer
 */
 
 #ifdef LTC_HMAC
 
-#define HMAC_BLOCKSIZE hash_descriptor[hash].blocksize
+#define LTC_HMAC_BLOCKSIZE hash_descriptor[hash].blocksize
 
 /**
    Initialize an HMAC context.
-   @param hmac     The HMAC state 
+   @param hmac     The HMAC state
    @param hash     The index of the hash you want to use 
    @param key      The secret key
    @param keylen   The length of the secret key (octets)
@@ -50,30 +48,28 @@ int hmac_init(hmac_state *hmac, int hash, const unsigned char *key, unsigned lon
     }
 
     /* allocate memory for key */
-    hmac->key = XMALLOC(HMAC_BLOCKSIZE);
+    hmac->key = XMALLOC(LTC_HMAC_BLOCKSIZE);
     if (hmac->key == NULL) {
        return CRYPT_MEM;
     }
 
     /* (1) make sure we have a large enough key */
-    if(keylen > HMAC_BLOCKSIZE) {
-        z = HMAC_BLOCKSIZE;
+    if(keylen > LTC_HMAC_BLOCKSIZE) {
+        z = LTC_HMAC_BLOCKSIZE;
         if ((err = hash_memory(hash, key, keylen, hmac->key, &z)) != CRYPT_OK) {
            goto LBL_ERR;
         }
-        if(hashsize < HMAC_BLOCKSIZE) {
-            zeromem((hmac->key) + hashsize, (size_t)(HMAC_BLOCKSIZE - hashsize));
-        }
         keylen = hashsize;
     } else {
         XMEMCPY(hmac->key, key, (size_t)keylen);
-        if(keylen < HMAC_BLOCKSIZE) {
-            zeromem((hmac->key) + keylen, (size_t)(HMAC_BLOCKSIZE - keylen));
-        }
     }
 
-    /* Create the initial vector for step (3) */
-    for(i=0; i < HMAC_BLOCKSIZE;   i++) {
+        if(keylen < LTC_HMAC_BLOCKSIZE) {
+            zeromem((hmac->key) + keylen, (size_t)(LTC_HMAC_BLOCKSIZE - keylen));
+        }
+
+    /* Create the initialization vector for step (3) */
+    for(i=0; i < LTC_HMAC_BLOCKSIZE;   i++) {
        buf[i] = hmac->key[i] ^ 0x36;
     }
 
@@ -82,7 +78,7 @@ int hmac_init(hmac_state *hmac, int hash, const unsigned char *key, unsigned lon
        goto LBL_ERR;
     }
 
-    if ((err = hash_descriptor[hash].process(&hmac->md, buf, HMAC_BLOCKSIZE)) != CRYPT_OK) {
+    if ((err = hash_descriptor[hash].process(&hmac->md, buf, LTC_HMAC_BLOCKSIZE)) != CRYPT_OK) {
        goto LBL_ERR;
     }
     goto done;
@@ -91,7 +87,7 @@ LBL_ERR:
     XFREE(hmac->key);
 done:
 #ifdef LTC_CLEAN_STACK
-   zeromem(buf, HMAC_BLOCKSIZE);
+   zeromem(buf, LTC_HMAC_BLOCKSIZE);
 #endif
  
    return err;    
@@ -99,6 +95,6 @@ done:
 
 #endif
 
-/* $Source: /cvs/libtom/libtomcrypt/src/mac/hmac/hmac_init.c,v $ */
-/* $Revision: 1.5 $ */
-/* $Date: 2006/11/03 00:39:49 $ */
+/* ref:         $Format:%D$ */
+/* git commit:  $Format:%H$ */
+/* commit time: $Format:%ai$ */
