@@ -1,18 +1,19 @@
+#include <tommath_private.h>
+#ifdef BN_MP_SUBMOD_C
 /* LibTomMath, multiple-precision integer library -- Tom St Denis
  *
- * LibTomMath is library that provides for multiple-precision
+ * LibTomMath is a library that provides multiple-precision
  * integer arithmetic as well as number theoretic functionality.
  *
- * The library is designed directly after the MPI library by
+ * The library was designed directly after the MPI library by
  * Michael Fromberger but has been written from scratch with
  * additional optimizations in place.
  *
  * The library is free for all purposes without any express
  * guarantee it works.
  *
- * Tom St Denis, tomstdenis@iahu.ca, http://math.libtomcrypt.org
+ * Tom St Denis, tstdenis82@gmail.com, http://libtom.org
  */
-#include <tommath.h>
 
 /* d = a - b (mod c) */
 int
@@ -34,3 +35,8 @@ mp_submod (mp_int * a, mp_int * b, mp_int * c, mp_int * d)
   mp_clear (&t);
   return res;
 }
+#endif
+
+/* ref:         $Format:%D$ */
+/* git commit:  $Format:%H$ */
+/* commit time: $Format:%ai$ */
